#!/usr/bin/env python3
"""
DHCP Packet Listener
Parameters:
    - IP: Network interface IP to listen on
Requires: scapy package and admin/root privileges
"""
from scapy.all import *
import argparse
from datetime import datetime

class DHCPListener:
    def __init__(self, interface_ip):
        self.interface_ip = interface_ip
        self.dhcp_messages = {
            1: "DISCOVER",
            2: "OFFER",
            3: "REQUEST",
            4: "DECLINE",
            5: "ACK",
            6: "NAK",
            7: "RELEASE",
            8: "INFORM"
        }
        
    def get_dhcp_options(self, packet):
        options = {}
        for option in packet[DHCP].options:
            if isinstance(option, tuple) and len(option) >= 2:
                options[option[0]] = option[1]
        return options
        
    def packet_callback(self, packet):
        if DHCP in packet:
            timestamp = datetime.now()
            message_type = None
            requested_addr = None
            hostname = None
            
            # Get DHCP options
            options = self.get_dhcp_options(packet)
            
            if 'message-type' in options:
                message_type = self.dhcp_messages.get(options['message-type'], "UNKNOWN")
            if 'requested_addr' in options:
                requested_addr = options['requested_addr']
            if 'hostname' in options:
                hostname = options['hostname'].decode() if isinstance(options['hostname'], bytes) else options['hostname']
            
            print(f"\n[{timestamp}] DHCP {message_type}")
            print(f"Source MAC: {packet[Ether].src}")
            if requested_addr:
                print(f"Requested IP: {requested_addr}")
            if hostname:
                print(f"Hostname: {hostname}")
            if 'server_id' in options:
                print(f"DHCP Server: {options['server_id']}")
            print("-" * 50)
    
    def start_listening(self):
        try:
            print(f"Starting DHCP packet listener on interface with IP {self.interface_ip}")
            print("Press Ctrl+C to stop...\n")
            
            # Set up filter for DHCP packets
            dhcp_filter = "udp and (port 67 or port 68)"
            sniff(filter=dhcp_filter, prn=self.packet_callback)
            
        except KeyboardInterrupt:
            print("\nListener stopped by user")
        except Exception as e:
            print(f"\nError: {str(e)}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="DHCP Packet Listener")
    parser.add_argument("ip", help="Network interface IP to listen on")
    args = parser.parse_args()
    
    listener = DHCPListener(args.ip)
    listener.start_listening()
